use std::sync::Arc;

use post::{
    adapter::gateway::post_repository_impl::PostRepositoryImpl,
    application::use_case::{
        get_all_post_info_use_case::{GetAllPostInfoUseCase, GetAllPostInfoUseCaseImpl},
        get_full_post_use_case::{GetFullPostUseCase, GetFullPostUseCaseImpl},
    },
    framework::db::post_db_service_impl::PostDbServiceImpl,
};
use sqlx::{Pool, Postgres};

pub struct Container {
    pub get_all_post_info_use_case: Arc<dyn GetAllPostInfoUseCase>,
    pub get_full_post_use_case: Arc<dyn GetFullPostUseCase>,
}

impl Container {
    pub fn new(db_pool: Arc<Pool<Postgres>>) -> Self {
        let post_db_service = Arc::new(PostDbServiceImpl::new(db_pool));

        let post_repository = Arc::new(PostRepositoryImpl::new(post_db_service.clone()));

        let get_all_post_info_use_case =
            Arc::new(GetAllPostInfoUseCaseImpl::new(post_repository.clone()));
        let get_full_post_use_case = Arc::new(GetFullPostUseCaseImpl::new(post_repository.clone()));

        Self {
            get_all_post_info_use_case,
            get_full_post_use_case,
        }
    }
}
